/*
 *	KDE3 Konx Style (version 0.1)
 *	Copyright (C) 2004, Martin Stubenschrott <stubenschrott@gmx.net>
 *
 *	Based on the great dotNET style written by:
 *		Chris Lee <clee@kde.org>
 *		Carsten Pfeiffer <pfeiffer@kde.org>
 *		Karol Szwed <gallium@kde.org>
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License version 2 as published by the Free Software Foundation.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public License
 *	along with this library; see the file COPYING.LIB.  If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 */
#include <qstyleplugin.h>
#include <qstylefactory.h>
#include <qpointarray.h>
#include <qpainter.h>
#include <qtabbar.h>
#include <qprogressbar.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qscrollbar.h>
#include <qscrollview.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qdrawutil.h>
#include <qapplication.h>
#include <qvariant.h>
#include <qpixmapcache.h>
#include <qslider.h>
#include <qsettings.h>
#include <kpixmap.h>

#include "konx.h"
#include "konx.moc"

#define DOTTEDLINE_WIDTH 3 // between each dot of a dotted line is DOTTEDLINE_WIDTH -1 pixels whitespace

/*	This code registers the style with QStyleFactory, which makes it possible
 *	for any code that uses QStyle to find out about it. */
// -- Style Plugin Interface -------------------------
class KonxStylePlugin : public QStylePlugin
{
	public:
		KonxStylePlugin() {}
		~KonxStylePlugin() {}

		QStringList keys() const {
			return QStringList() << "Konx";
		}

		QStyle* create( const QString& key ) {
			if (key == "konx")
				return new KonxStyle;
			return 0;
		}
};

Q_EXPORT_PLUGIN( KonxStylePlugin )

// ---------------------------------------------------

KonxStyle::KonxStyle() :
	KStyle( AllowMenuTransparency ),
	kickerMode(false),
	toolbarBlendWidget(0),
	kdeToolbarWidget("kde toolbar widget")
{
	winstyle = 0L;
	winstyle = QStyleFactory::create("Windows");
	
	//	We don't have the Windows style, neither builtin nor as a plugin.
	//	Use any style rather than crashing.
	if (winstyle == 0L)
		winstyle = QStyleFactory::create("B3");

//     if (qApp->inherits("KApplication")) 
//         connect( qApp, SIGNAL( kdisplayPaletteChanged() ), SLOT( paletteChanged() ));

	QSettings settings;
	useAlternativeColors = settings.readBoolEntry("/KStyle/Settings/UseAlternativeColors", true);
	reverseLayout = QApplication::reverseLayout();
}

KonxStyle::~KonxStyle()
{
}

// stolen completely from dotNET
bool KonxStyle::inheritsKHTML(const QWidget* w) const
{
	if (w->parentWidget(true) && w->parentWidget(true)->parentWidget(true) && w->parentWidget(true)->parentWidget(true)->parentWidget(true) &&  w->parentWidget(true)->parentWidget(true)->parentWidget(true)->inherits("KHTMLView"))
		return true;
	else
		return false;
}

void KonxStyle::polish(QWidget* widget)
{
	if (!qstrcmp(qApp->argv()[0], "kicker") || widget->inherits("Kicker"))
		kickerMode = true;

	if (widget->isTopLevel())
		return;

	// for focus highlight
	if (dynamic_cast<QLineEdit*>(widget) ||
		dynamic_cast<QSpinWidget*>(widget) ||
		dynamic_cast<QScrollView*>(widget))
	{
        widget->installEventFilter(this);
    }

	winstyle->polish(widget);
}

void KonxStyle::unPolish(QWidget* widget)
{
	// for focus highlight
	if (dynamic_cast<QLineEdit*>(widget) ||
		dynamic_cast<QSpinWidget*>(widget) ||
		dynamic_cast<QScrollView*>(widget))
	{
        widget->removeEventFilter(this);
    }

	winstyle->unPolish(widget);
}

/*	This is the code that renders 90+% of the buttons in Konx.
 *	Changing the code here will most likely affect EVERYTHING.
 *	If you want to modify the style, this is a good place to start.
 *	Also, take a look at the sizeMetric function and play with the
 *	widths that things return for different Frame elements.
 */
void KonxStyle::renderButton(QPainter *p,
                               const QRect &r,
                               const QColorGroup &g,
                               bool sunken) const
{
	p->save();
	
	if (sunken)
	{
		p->setPen(g.highlight().dark());
		p->setBrush(g.highlight());
		p->drawRect(r);
	}
	else
	{
		int x, y, w, h;
		r.rect(&x, &y, &w, &h);

		p->setPen(g.button().dark());
		if (useAlternativeColors)
			p->setBrush(g.midlight());
		else
			p->setBrush(g.button());

		p->drawRect(r);

		p->setPen(g.button().light());
		p->drawLine(x + 1, y + 1, x + 1, y + h - 2);
		p->drawLine(x + 1, y + 1, x + w - 2, y + 1);
		p->setPen(g.button().dark(115));
		p->drawLine(x + 2, y + h - 2, x + w - 2, y + h - 2);
		p->drawLine(x + w - 2, y + 2, x + w - 2, y + h - 2);
	}

	p->restore();
}

/*	This is the code that renders the slider handles on scrollbars. It draws
 *	them just like normal buttons, by calling renderButton, but it also adds
 *	very subtle grips to the slider handle.
 */
void KonxStyle::renderSlider(QPainter *p,
                               const QRect &r,
                               const QColorGroup &cg,
							   const bool horiz) const
{
//     int x, y, x2, y2;
//     r.coords(&x,&y,&x2,&y2);
	QRect smaller_rect = r;
	if (horiz)
		smaller_rect.addCoords(0, 1, 0, -1);
	else
		smaller_rect.addCoords(1, 0, -1, 0);

	p->save();

	// need to paint this special area in midlight color like the rest of the scrollbar
	p->setBrush( NoBrush );
	p->setPen( cg.midlight() );
	p->drawRect( r );

	// only draw grips when large enough
	if ( horiz && (r.width()) > 24 ||
		!horiz && (r.height()) > 24)
	{
		// 'horiz ? :' is swapped intentionally, don't ask why
		drawPrimitive(PE_DockWindowResizeHandle, p, smaller_rect, cg, (horiz ? Style_Default : Style_Horizontal));
	}
	else //no grippers
		p->setBrush(cg.button());

	p->setPen(cg.mid());
	p->drawRect(smaller_rect);
	/* disabled for now
	if (horiz)
		p->drawRect(x, y+1, x2-x, y2-y-2);
	else
		p->drawRect(x+1, y, x2-x-2, y2-y);
	
	p->setPen(cg.dark());
	if (horiz)
	{
		p->drawLine(x, y, x, y2);
		p->drawLine(x2, y, x2, y2);
	}
	else
	{
		p->drawLine(x, y, x2, y);
		p->drawLine(x, y2, x2, y2);
	}
	*/

	p->restore();
}

/*	This function does the rendering for most of the panels in Konx.
 *	The QCOORD stuff is used to provide the slightly-rounded corners.
 *	Changing this function will have pretty widespread effects. Also a good
 *	place to start if you're looking to make your own style.
 */
void KonxStyle::renderPanel(QPainter *p,
                              const QRect &r,
                              const QColorGroup &g,
                              bool sunken,
                              bool thick,
							  bool enabled,
							  bool focused) const
{
	int x, x2, y, y2, w, h;
	r.rect(&x,&y,&w,&h);
	r.coords(&x, &y, &x2, &y2);

	p->save();

	if (sunken)
	{
		QRect inner = r;
		inner.addCoords(1,1,-1,-1);

		// draw focus highlight
		if (focused && enabled && thick)
		{
			p->setPen(g.highlight().light(110));
			p->drawRect(r);
			p->setPen(g.highlight().dark(120));
			p->drawRect(inner);
			p->setPen(g.highlight().dark(150));
			p->drawLine(inner.topLeft(), inner.topRight());
		}
		else
		{
			if (thick)
			{
				p->setPen(g.background());
				p->drawRect(r);

				p->setPen(g.mid());
				p->drawRect(inner);
				if (enabled)
				{
					p->setPen(g.dark());
					p->drawLine(inner.topLeft(), inner.topRight());
				}
			}
			else
			{
				p->setPen(g.mid());
				p->drawRect(r);
				if (enabled)
				{
					p->setPen(g.dark());
					p->drawLine(r.topLeft(), r.topRight());
				}
			}
		}
	}
	else
	{
		const QCOORD oCorners[] = { x, y2, x, y, x2, y, x2, y2 };
		const QCOORD iCorners[] = { x+1, y2-1, x+1, y+1, x2-1, y+1, x2-1, y2-1 };
		p->setPen(g.background().dark());
		p->drawConvexPolygon(QPointArray(4, oCorners));

		if (thick)
		{
			p->setPen(g.background().dark(115));
			p->drawConvexPolygon(QPointArray(4, iCorners));
			p->setPen(g.background().light());

			p->drawLine(x+1, y+1, x+1, h-2);
			p->drawLine(x+1, y+1, w-2, y+1);
		}
	}


	//	Stolen wholesale from dotNET, which stole it from Keramik.
	//	I don't like it, but oh well.
	if (kickerMode)
	{
		if (!thick)
		{
			if (sunken)
			{
				const QCOORD corners[] = { x2, y, x2, y2, x, y2, x, y };
				p->setPen(g.background().dark());
				p->drawConvexPolygon(QPointArray(4, corners));
				p->setPen(g.background().light());
				p->drawPolyline(QPointArray(4, corners), 0, 3);
			}
			else
			{
				const QCOORD corners[] = { x, y2, x, y, x2, y, x2, y2 };
				p->setPen(g.background().dark());
				p->drawPolygon(QPointArray(4, corners));
				p->setPen(g.background().light());
				p->drawPolyline(QPointArray(4, corners), 0, 3);
			}
		}
	}

	p->restore();
}


void KonxStyle::drawKStylePrimitive(KStylePrimitive kpe,
                                      QPainter *p,
                                      const QWidget* widget,
                                      const QRect &r,
                                      const QColorGroup &cg,
                                      SFlags flags,
                                      const QStyleOption& opt) const
{
	int x,y,w,h,x2,y2;
	r.rect (&x,&y,&w,&h);
	x2 = x+w-1; y2 = y+h-1;

	switch( kpe )
	{
		//  SLIDER
		//  ------
		case KPE_SliderGroove:
		{
			const QSlider *slider = (const QSlider*)widget;
			bool horizontal = slider->orientation() == Horizontal;

			p->save();

			renderPanel(p, r, cg, true, false, flags & Style_Enabled);
			p->fillRect(x+1, y+1, w-2, h-2, 
					(flags & Style_Enabled) ? cg.midlight() : cg.button());

			// draw some small vertical(or horizontal) lines
			if (h >= 10 && w >= 10)
			{
				p->setPen(cg.mid());
				if (horizontal)
					for (int i = 10; i < w - 5; i += 10)
						p->drawLine(x + i, y + 5, x + i, h - 6);
				else
					for (int i = 10; i < h - 5; i += 10)
						p->drawLine(x + 5, y + i, w - 6, y + i);
			}

			p->restore();

			break;
		}

		case KPE_SliderHandle:
		{
			const QSlider *slider = (const QSlider*)widget;
			bool horizontal = slider->orientation() == Horizontal;
			int cx = x + (w/2);
			int cy = y + (h/2);

			p->save();

			p->setBrush(cg.highlight());
			p->setPen(cg.dark());
			p->drawRect(r);

			if (horizontal)
			{
				// draw gripper
				if (h >= 21)
				{
					p->drawLine(cx-3, cy-6, cx+3, cy-6);
					p->drawLine(cx-2, cy-3, cx+2, cy-3);
					p->drawLine(cx-3, cy  , cx+3, cy  );
					p->drawLine(cx-2, cy+3, cx+2, cy+3);
					p->drawLine(cx-3, cy+6, cx+3, cy+6);
				}
				else if (h >= 11)
				{
					p->drawLine(cx-3, cy-3, cx+3, cy-3);
					p->drawLine(cx-2, cy  , cx+2, cy  );
					p->drawLine(cx-3, cy+3, cx+3, cy+3);
				}
			}
			else // vertical
			{
				// draw gripper
				if (w >= 21)
				{
					p->drawLine(cx-6, cy-3, cx-6, cy+3);
					p->drawLine(cx-3, cy-2, cx-3, cy+2);
					p->drawLine(cx  , cy-3, cx  , cy+3);
					p->drawLine(cx+3, cy-2, cx+3, cy+2);
					p->drawLine(cx+6, cy-3, cx+6, cy+3);

				}
				else if (w >= 11)
				{
					p->drawLine(cx-3, cy-3, cx-3, cy+3);
					p->drawLine(cx  , cy-2, cx  , cy+2);
					p->drawLine(cx+3, cy-3, cx+3, cy+3);
				}
			}

			p->restore();

			break;
		}

		default:
			KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);
	}
}


// This function draws primitive elements as well as their masks.
void KonxStyle::drawPrimitive(PrimitiveElement pe,
                                QPainter *p,
                                const QRect &r,
                                const QColorGroup &cg,
                                SFlags flags,
                                const QStyleOption &opt ) const
{
	bool down   = flags & Style_Down;
	bool on     = flags & Style_On;
	bool sunken = flags & Style_Sunken;
	bool horiz  = flags & Style_Horizontal;
	bool enabled  = flags & Style_Enabled;
	bool focused  = flags & Style_HasFocus;

	int x, x2, y, y2, w, h;
	r.rect(&x, &y, &w, &h);
	r.coords(&x, &y, &x2, &y2);

	switch(pe)
	{
		case PE_HeaderSection:
		{
			if (useAlternativeColors)
				p->fillRect(r, QBrush(cg.midlight()));
			else
				p->fillRect(r, QBrush(cg.button()));

			p->setPen(cg.mid());
			p->drawLine(x, y2, x2, y2);
			p->drawLine(x2, y, x2, y2);
			break;
	   	}

		//	BUTTONS
		//	-------
		case PE_ButtonBevel: 
		case PE_ButtonTool:
		case PE_ButtonDropDown:
		case PE_ButtonCommand:
		{

			if (on || down)
			{
				renderButton(p, r, cg, true);
				p->setPen( cg.highlightedText() );
			}
			else if (!enabled)
			{
				p->setPen(cg.mid());
				if (useAlternativeColors)
					p->setBrush(cg.midlight());
				else
					p->setBrush(cg.button());
				p->drawRect(r);
			}
			else
			{
				renderButton(p, r, cg, false);
			}

			break;
		}

		case PE_ButtonDefault:
		{
			p->setPen(cg.highlight());
			p->drawRect(r);
			break;
		}

		case PE_ScrollBarSlider:
		{
			renderSlider(p, r, cg, horiz);
			break;
		}

		case PE_ScrollBarAddPage:
		case PE_ScrollBarSubPage:
		{
			if (on || down)
			{
				p->fillRect(r, QBrush(cg.mid().dark()));
				if (useAlternativeColors)
					p->setPen (cg.midlight());
				else
					p->setPen (cg.button());
                p->drawLine(r.topLeft(), horiz ? r.topRight() : r.bottomLeft());
                p->drawLine(r.bottomRight(), horiz ? r.bottomLeft() : r.topRight());
			}
			else
				if (useAlternativeColors)
					p->fillRect(r, QBrush(cg.midlight()));
				else
					p->fillRect(r, QBrush(cg.button()));
			break;
		}

		//	SCROLLBAR BUTTONS
		//	-----------------
		case PE_ScrollBarAddLine:
		case PE_ScrollBarSubLine:
		{
			if (on || down)
			{
				p->fillRect(r, QBrush(cg.highlight()));
				if (useAlternativeColors)
					p->setPen (cg.midlight());
				else
					p->setPen (cg.button());
				p->drawRect(r);
			}
			else
				if (useAlternativeColors)
					p->fillRect(r, QBrush(cg.midlight()));
				else
					p->fillRect(r, QBrush(cg.button()));
			
			p->setPen(down ? cg.highlightedText() : cg.foreground());
			if (pe == PE_ScrollBarAddLine)
				drawPrimitive((horiz ? PE_ArrowRight : PE_ArrowDown), p, r, cg, flags);
			else
				drawPrimitive((horiz ? PE_ArrowLeft : PE_ArrowUp), p, r, cg, flags);

			break;
		}

		//	CHECKBOXES
		//	----------
		case PE_Indicator:
		{
			p->setPen( NoPen );
			p->setBrush(flags & Style_Enabled ? cg.base() : cg.background());
			p->drawRect(r);
			renderPanel(p, r, cg, true, false, enabled);
			break;
		}

		case PE_IndicatorMask:
		{
			p->fillRect (r, color1);
			break;
		}

		//	RADIOBUTTONS
		//	------------
		case PE_ExclusiveIndicator:
		{
			const QCOORD outline[] = { 0, 7, 0, 5, 1, 4, 1, 3, 2, 2, 3, 1, 4, 1, 5, 0, 7, 0, 8, 1, 9, 1, 10, 2, 11, 3, 11, 4,
									   12, 5, 12, 7, 11, 8, 11, 9, 10, 10, 9, 11, 8, 11, 7, 12, 5, 12, 4, 11, 3, 11, 2, 10, 1, 9, 1, 8 };
			QPointArray pa = QPointArray(28, outline);

			p->fillRect(r, cg.background());
			p->setPen(cg.background().dark());
			p->setBrush(flags & Style_Enabled ? cg.base() : cg.background());
			p->drawPolygon(pa, 28);
			// draw the brighter part of the circle
			p->setPen(cg.mid());
			p->drawLineSegments(pa, 16, -1);
			// the bottom-center (and some other) points are still in the wrong color
			p->drawPoint(6,12);
			p->drawPoint(0,6);
			p->drawPoint(0,7);
			p->drawPoint(12,6);
			p->drawPoint(12,7);

			break;
		}

		case PE_ExclusiveIndicatorMask:
		{
			const QCOORD outline[] = { 0, 7, 0, 5, 1, 4, 1, 3, 2, 2, 3, 1, 4, 1, 5, 0, 7, 0, 8, 1, 9, 1, 10, 2, 11, 3, 11, 4, 12, 5, 12, 7, 11, 8, 11, 9, 10, 10, 9, 11, 8, 11, 7, 12, 5, 12, 4, 11, 3, 11, 2, 10, 1, 9, 1, 8 };

			p->fillRect(r, color0);
			p->setPen(color1);
			p->setBrush(color1);
			p->drawPolygon(QPointArray(28, outline));
			break;
		}

	//	GENERAL PANELS
	//	--------------

		// default winstyle group boxes
		// case PE_PanelGroupBox:
		// case PE_GroupBoxFrame:

		case PE_WindowFrame:
		case PE_Panel:
		{
			renderPanel(p, r, cg, sunken, true, enabled, focused);
			break;
		}

		case PE_PanelLineEdit:
		{
			renderPanel(p, r, cg, true, true, enabled, focused);
			break;
//             QRect inner = r;
//             inner.addCoords(1,1,-1,-1);
//             renderPanel(p, inner, cg, true, true, enabled);
//             
//             break;
		}

		case PE_StatusBarSection:
		{
			renderPanel(p, r, cg, true, false, false, false);
			break;
		}

		case PE_TabBarBase:	// Still not sure what this one does
		case PE_PanelTabWidget:
		{
			renderPanel(p, r, cg, false);
			break;
		}

		case PE_PanelPopup:
		{
			p->setPen(cg.midlight().light(105));
			p->drawLine(r.topLeft(), r.topRight());
			p->drawLine(r.topLeft(), r.bottomLeft());
			p->setPen(cg.dark());
			p->drawLine(r.topRight(), r.bottomRight());
			p->drawLine(r.bottomLeft(), r.bottomRight());
			break;
		}

		//	MENU / TOOLBAR PANEL
		//	--------------------
		case PE_PanelMenuBar: 			// Menu
		case PE_PanelDockWindow:		// Toolbar
		{
			p->setPen(cg.button());
			p->setBrush(cg.button());
			p->drawRect(r);
			break;
		}

		//	TOOLBAR/DOCK WINDOW/SPLITTER HANDLE
		//	-----------------------------------
		case PE_Splitter: 
			if (horiz)
			{
				y = r.center().y() - 9;
				h = 20;
			}
			else
			{
				x = r.center().x() - 9;
				w = 20;
			}
			/* fall through */
		case PE_DockWindowHandle:
		{
			p->fillRect(r, cg.button());

			int j = r.center().x();
			int k = r.center().y();
			p->setPen(cg.dark());
			if (horiz)
			{
				for (int i=y+1; i <= y+h-1; i+=DOTTEDLINE_WIDTH)
					p->drawLine(j - 1, i, j + 1, i);
			}
			else
			{
				for (int i=x+2; i <= x+w-1; i+=DOTTEDLINE_WIDTH)
					p->drawLine(i - 1, k - 1, i - 1, k + 1);
			}

			break;
		}
		/* also used for the gripper of scroll bars */
		case PE_DockWindowResizeHandle:
		{
			p->fillRect(r, cg.button());

			int j = r.center().x();
			int k = r.center().y();
			if (horiz)
			{
				p->setPen(cg.dark());
				p->drawLine(j - 2, k - 6, j + 3, k - 6);
				p->drawLine(j - 2, k    , j + 3, k	);
				p->drawLine(j - 2, k + 6, j + 3, k + 6);

				p->setPen(cg.mid().dark(110));
				p->drawLine(j - 1, k - 3, j + 2, k - 3);
				p->drawLine(j - 1, k + 3, j + 2, k + 3);
			}
			else
			{
				p->setPen(cg.dark());
				p->drawLine(j - 6, k - 2, j - 6, k + 3);
				p->drawLine(j    , k - 2, j    , k + 3);
				p->drawLine(j + 6, k - 2, j + 6, k + 3);

				p->setPen(cg.mid().dark(110));
				p->drawLine(j - 3, k - 1, j - 3, k + 2);
				p->drawLine(j + 3, k - 1, j + 3, k + 2);
			}

			break;
		}


		//	TOOLBAR SEPARATOR
		//	-----------------
		case PE_DockWindowSeparator:
		{
			QPen dotPen = cg.dark();
			p->setPen( dotPen );

			int j = r.center().x();
			int k = r.center().y();
			if (horiz)
				for (int i=y+1; i <= y+h-1; i+=3)
					p->drawPoint(j, i);
			else
				for (int i=x+1; i <= x+w-1; i+=3)
					p->drawPoint(i, k);
			break;
		}

		case PE_CheckMark:
		{
			int x = r.center().x() - 3, y = r.center().y() - 3;
			QPen pen = p->pen();
			if (flags & Style_Enabled)
				if (flags & Style_Down)
					pen.setColor( cg.highlight() );
				else
					pen.setColor( QColor(0,0,192) );
			else
				pen.setColor( cg.mid() );

          	pen.setWidth (2);
			p->setPen( pen);
			p->setBrush(flags & Style_Down ? cg.highlight() : pen.color());

			if (flags & Style_On || flags & Style_Down)
			{
				p->drawLine (x,y,x+6, y+6);
				p->drawLine (x,y+6,x+6, y);
				// fix missing points at the end of the lines
				p->drawPoint (x+6, y);
				p->drawPoint (x+6, y+6);
			}
			else if (flags & Style_Off) // empty
			{
			}
			else	//tristate
			{
				p->setPen (NoPen);
				p->drawRect (x,y,x+4, y+4);
			}

			break;
		}

		case PE_SpinWidgetUp:
		case PE_SpinWidgetDown:
		case PE_HeaderArrow:
		case PE_ArrowUp:
		case PE_ArrowDown:
		case PE_ArrowLeft:
		case PE_ArrowRight:
		{
			QPointArray a;

			switch (pe)
			{
				case PE_SpinWidgetUp:
				case PE_ArrowUp:
				{
					a.setPoints(7, u_arrow);
					break;
				}
				case PE_SpinWidgetDown:
				case PE_ArrowDown:
				{
					a.setPoints(7, d_arrow);
					break;
				}
				case PE_ArrowLeft:
				{
					a.setPoints(7, l_arrow);
					break;
				}
				case PE_ArrowRight:
				{
					a.setPoints(7, r_arrow);
					break;
				}
				default:
				{
					if (flags & Style_Up) 
						a.setPoints(7, u_arrow);
					else 
						a.setPoints(7, d_arrow);
				}
			}

			p->save();

			if (flags & Style_Down)
				p->translate(pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical));

			a.translate((r.x() + r.width()/2), (r.y() + r.height()/2));
			if (p->pen() == QPen::NoPen) 
			{
				if (flags & Style_Enabled)
					p->setPen(cg.buttonText());
				else
					p->setPen(cg.mid());
			}

			p->drawLineSegments(a, 0, 3);
			p->drawPoint(a[6]);

			p->restore();
			break;
		}

		default:
			return winstyle->drawPrimitive(pe, p, r, cg, flags, opt);
	}
}


void KonxStyle::drawControl(ControlElement element,
                              QPainter *p,
                              const QWidget *widget,
                              const QRect &r,
                              const QColorGroup &cg,
                              SFlags flags,
                              const QStyleOption& opt) const
{
	switch (element)
	{
		//	CHECKBOXES
		//	----------
		case CE_CheckBox:
		{
			drawPrimitive(PE_Indicator, p, r, cg, flags);
			p->setPen(cg.foreground());
			drawPrimitive(PE_CheckMark, p, r, cg, flags);
			break;
		}

		//	PROGRESSBAR
		//	-----------
		case CE_ProgressBarGroove:
		{
			renderPanel(p, r, cg, true, false, flags & Style_Enabled);
			break;
		}

		case CE_ProgressBarContents:
		{
			const QProgressBar *pb = (const QProgressBar *)widget;

			if (pb->totalSteps())
			{
				int x, y, w, h;
				double percent = (double)pb->progress() / (double)pb->totalSteps();

				r.rect(&x, &y, &w, &h);
				int total_width = w;

				p->fillRect(x + 1, y + 1, w - 2, h - 2, 
					(flags & Style_Enabled) ? cg.midlight() : cg.button());

				if (reverseLayout)
				{
					x += w - ((int)(w * percent));
					y += 1; h -= 2;
					w = ((int)(w * percent)) - 2;
				}
				else
				{
					x += 1; y += 1; h -= 2;
					w = (int)(w * percent) - 2;
				}

				p->fillRect(x, y, w, h, cg.highlight());

				// the vertical line at the right side of the progress bar value
				p->setPen(cg.highlight().dark(120));
				if (pb->progress() < pb->totalSteps())
				{
					if (reverseLayout)
						p->drawLine(x - 1, y, x - 1, h);
					else
						p->drawLine(x + w, y, x + w, h);
				}


				// draw some small vertical lines
				if (h >= 10)
				{
					bool light_pen_set = false; //only set the new pen once
					for (int i = 10; i < total_width - 5; i += 10)
					{
						if (i > w && !light_pen_set)
						{
							light_pen_set = true;
							p->setPen(cg.mid());
						}
						p->drawLine(x + i, y + 5, x + i, h - 5);
					}
				}

			}
			break;
		}

		//	RADIOBUTTONS
		//	------------
		case CE_RadioButton:
		{
			drawPrimitive(PE_ExclusiveIndicator, p, r, cg, flags);

			if (flags & Style_On || flags & Style_Down)
			{
				QCOORD center[] = { 4, 5, 4, 7, 5, 8, 7, 8, 8, 7, 8, 5, 7, 4, 5, 4 };
				QPointArray c(8, center);
				p->setPen(flags & Style_Down ? cg.highlight() :
						  flags & Style_Enabled ? QColor(255,128,0) : cg.mid());
				p->setBrush(flags & Style_Down ? cg.highlight() :
						  flags & Style_Enabled ? QColor(255,128,0) : cg.mid());
				p->drawPolygon(c);
			}
			break;
		}

		//	TABS
		//	----
		case CE_TabBarTab:
		{
			const QTabBar * tb = (const QTabBar *) widget;
			QTabBar::Shape tbs = tb->shape();
			bool selected = flags & Style_Selected;
			int x, x2, y, y2, w, h;
			r.rect(&x, &y, &w, &h);
			r.coords(&x, &y, &x2, &y2);

			switch (tbs)
			{
				case QTabBar::RoundedAbove:
				{
					y2 -= 1;

					p->setPen(cg.background().dark());
					if (selected)
					{
						const QCOORD oCorners[] = { x, y2, x, y, x2, y, x2, y2 };
						p->drawPolyline(QPointArray(4, oCorners));

						p->setPen(cg.background().dark(115));
						/* Right inner border */
						p->drawLine(x + w - 2, y + 1, x + w - 2, y + h - 2);
						p->setPen(cg.background().light());
						/* Top inner border */
						p->drawLine(x + 1, y + 1, x + w - 2, y + 1);
						/* Left inner border */
						p->drawLine(x + 1, y + 1, x + 1, y + h - 1);
						// draw focus rectangle
						if (tb->hasFocus())
							drawPrimitive(PE_FocusRect, p, QRect(x + 3, y + 3, x2 - x - 5, y + h - 5), cg);

					}
					else
					{
						QRect r2(x+1, y+3, w-2, h-5);
						p->fillRect(r2, cg.mid().light(105));

						const QCOORD oCorners[] = { x, y2, x, y + 2, x2, y + 2, x2, y2 };
						p->drawPolyline(QPointArray(4, oCorners));

						p->setPen(cg.background().light());
						/* Top of underlying tabbar-widget */
						p->drawLine(x, y + h - 1, x + w - 1, y + h - 1);
						p->setPen(cg.background().dark());
						/*	Top of underlying tabbar-widget */
						p->drawLine(x, y + h - 2, x + w - 1, y + h - 2);
						/*	Outer right border */
						p->drawLine(x + w - 1, y + 3, x + w - 1, y + h - 2);
					}

					if (opt.tab()->identifier() == 0)
					{
						p->setPen(cg.background().dark());
						p->drawPoint(x, y + h - 1);
					}
					break;
				}

				case QTabBar::RoundedBelow:
				{
					p->setPen(cg.background().dark());
					if (selected)
					{
						const QCOORD oCorners[] = { x, y + 1, x, y2, x2, y2, x2, y + 1 };
						p->drawPolyline(QPointArray(4, oCorners));

						p->setPen(cg.background().dark(115));
						p->drawLine(x + 1, y + h - 2, x + w - 2, y + h - 2);
						/*	Right inner border */
						p->drawLine(x + w - 2, y + 1, x + w - 2, y + h - 2);
						p->setPen(cg.background().light());
						/* Left inner border */
						p->drawLine(x + 1, y, x + 1, y + h - 2);
						// draw focus rectangle
						if (tb->hasFocus())
							drawPrimitive(PE_FocusRect, p, QRect(x + 3, y + 3, x2 - x - 5, y + h - 5), cg);

					}
					else
					{
						y2 -= 2;

						const QCOORD oCorners[] = { x, y, x, y2, x2, y2, x2, y };
						QRect r2(x + 1, y + 2, w - 2, h - 5);
						p->fillRect(r2, cg.mid().light(105));

						p->drawPolyline(QPointArray(4, oCorners));

						p->setPen(cg.mid().dark(115));
						/*	Inner right border */
						p->drawLine(x + w - 2, y + 2, x + w - 2, y + h - 4);
						p->drawLine(x + 1, y + h - 4, x + w - 2, y + h - 4);

						p->setPen(cg.background().dark(115));
						/* Top of underlying tabbar-widget */
						p->drawLine(x, y, x + w - 1, y);
						p->setPen(cg.background().dark());
						/*	Top of underlying tabbar-widget */
						p->drawLine(x, y + 1, x + w - 1, y + 1);
						/*	Outer right border */
						p->drawLine(x + w - 1, y + 3, x + w - 1, y + h - 2);

					}

					if (opt.tab()->identifier() == 0)
					{
						p->setPen(cg.background().dark());
						p->drawPoint(x, y);
					}
					break;
				}

				default:
					winstyle->drawControl(element, p, widget, r, cg, flags, opt);
			}

			break;
		}

		case CE_TabBarLabel:
		{
			const int text_flags = AlignVCenter | AlignHCenter | ShowPrefix |
			                       DontClip | SingleLine;

			p->save();

			// offset inactive tabs
			// ugly hack stolen from Baghira, since KTabWidget does not
			// set Style_Selected correctly (as of 3.3-beta2)
			// Normally: if (!(flags & Style_Selected))
			const QTabBar * tb = ( const QTabBar * ) widget;
							
			if (tb && tb->currentTab() != opt.tab()->identifier())
				p->translate(0, pixelMetric(PM_ButtonShiftVertical));

			p->setPen(flags & Style_Enabled ? cg.foreground() : cg.mid().dark());
			p->drawText(r, text_flags, opt.tab()->text());

			p->restore();
			break;
		}

		case CE_PushButton:
		{
			QPushButton *button = (QPushButton *)widget;
			QRect br = r;
			bool btnDefault = button->isDefault();

			// offset all buttons by 1 pixel on each side, so we can draw the default
			// button indicator for default buttons
			static int di = pixelMetric(PM_ButtonDefaultIndicator);
			br.addCoords(di, di, -di, -di);

			if (btnDefault)
				drawPrimitive(PE_ButtonDefault, p, r, cg, flags);

			drawPrimitive(PE_ButtonBevel, p, br, cg, flags);
			break;
		}


        case CE_PushButtonLabel:
        {
            int x, y, w, h;
            r.rect( &x, &y, &w, &h );

            const QPushButton* button = static_cast<const QPushButton *>( widget );
            bool active = button->isOn() || button->isDown();
            bool cornArrow = false;

            // Shift button contents if pushed.
            if ( active )
            {
                x += pixelMetric(PM_ButtonShiftHorizontal, widget);
                y += pixelMetric(PM_ButtonShiftVertical, widget);
                flags |= Style_Sunken;
            }

            // Does the button have a popup menu?
            if ( button->isMenuButton() )
            {
                int dx = pixelMetric( PM_MenuButtonIndicator, widget );
                if ( button->iconSet() && !button->iconSet()->isNull()  &&
                    (dx + button->iconSet()->pixmap (QIconSet::Small, QIconSet::Normal, QIconSet::Off ).width()) >= w )
                {
                    cornArrow = true; //To little room. Draw the arrow in the corner, don't adjust the widget
                }
                else
                {
                    drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - dx - 8, y + 2, dx, h - 4), r ),
                                cg, flags, opt );
                    w -= dx;
                }
            }

            // Draw the icon if there is one
            if ( button->iconSet() && !button->iconSet()->isNull() )
            {
                QIconSet::Mode  mode  = QIconSet::Disabled;
                QIconSet::State state = QIconSet::Off;

                if (button->isEnabled())
                    mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
                if (button->isToggleButton() && button->isOn())
                    state = QIconSet::On;

                QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );

                if (button->text().isEmpty() && !button->pixmap())
                    p->drawPixmap( x + w/2 - pixmap.width()/2, y + h / 2 - pixmap.height() / 2,
                                    pixmap );
                else
                    p->drawPixmap( x + 4, y + h / 2 - pixmap.height() / 2, pixmap );

                if (cornArrow) //Draw over the icon
                    drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - 6, x + h - 6, 7, 7), r ),
                                cg, flags, opt );


                int  pw = pixmap.width();
                x += pw + 4;
                w -= pw + 4;
            }

            // Make the label indicate if the button is a default button or not
            drawItem( p, QRect(x, y, w, h), AlignCenter|ShowPrefix, button->colorGroup(),
                        button->isEnabled(), button->pixmap(), button->text(), -1,
                        active ?  &button->colorGroup().highlightedText() : &button->colorGroup().buttonText() );


            if ( flags & Style_HasFocus )
                drawPrimitive( PE_FocusRect, p,
                                visualRect( subRect( SR_PushButtonFocusRect, widget ), widget ),
                                cg, flags );
            break;
        }

		//	MENUBAR ITEM (sunken panel on mouse over)
		//	-----------------------------------------
		case CE_MenuBarItem:
		{
			QMenuItem *mi = opt.menuItem();
			bool active  = flags & Style_Active;
			bool focused = flags & Style_HasFocus;
			const int text_flags = AlignVCenter | AlignHCenter | ShowPrefix | DontClip | SingleLine;
			int x, x2, y, y2;
			r.coords(&x, &y, &x2, &y2);

			if (active && focused)
			{
				p->setBrush(cg.highlight());
				p->setPen(cg.highlight().dark());
				p->drawRect(r);
			}

			p->setPen((active && focused ? cg.highlightedText() : cg.buttonText()));
			p->drawText(r, text_flags, mi->text());
			break;
		}


		//	POPUPMENU ITEM (highlighted on mouseover)
		//	------------------------------------------
		case CE_PopupMenuItem:
		{
			QMenuItem *mi = opt.menuItem();
			if (!mi)
				return;

			const QPopupMenu *pum = (const QPopupMenu *) widget;
			static const int itemFrame = 2;
			static const int itemHMargin = 3;
			static const int itemVMargin = 3;
			static const int rightBorder = 20;
			const int tab = opt.tabWidth();

			int checkcol = QMAX(opt.maxIconWidth(), 20);
			int x, x2, y, y2, w, h;
			r.rect(&x, &y, &w, &h);
			r.coords(&x, &y, &x2, &y2);

			bool active = flags & Style_Active;
			bool disabled = !mi->isEnabled();
			bool checkable = pum->isCheckable();
			bool etchtext = styleHint (SH_EtchDisabledText);

			QColor textcolor; // color of the menu item's text

			int xpos = x;
			int xm = itemFrame + checkcol + itemHMargin;

			if (pum->erasePixmap() && !pum->erasePixmap()->isNull())
				p->drawPixmap( x, y, *pum->erasePixmap(), x, y, w, h );
			else
			{
				// the highlight color differs in the inactive palette, but we need consistent colors here
				QColor colHighlight;
				QPalette pal = pum->palette();
				colHighlight = pal.active().color(QColorGroup::Highlight);

				int dim = pixelMetric(PM_MenuButtonIndicator) + 4;
				if (!reverseLayout)
				{
					if (useAlternativeColors)
						p->fillRect( x, y, x + w - dim, h, cg.midlight());
					else
						p->fillRect( x, y, x + w - dim, h, cg.button());
					p->fillRect( x+w-dim, y, dim, h, colHighlight);
				}
				else 
				{
					// FIXME:
					if (useAlternativeColors)
						p->fillRect( x+dim, y, w - dim, h, cg.midlight());
					else
						p->fillRect( x+dim, y, w - dim, h, cg.button());
					p->fillRect( x, y, dim, h, colHighlight );
				}
			}

			if (mi->isSeparator())
			{
				p->setPen(cg.highlight());
				// draw a dotted line as the separator
				if (!reverseLayout)
                     for (int i=x+26; i < x+w; i+=DOTTEDLINE_WIDTH)
						 p->drawPoint(i, y+1);
				else
                     for (int i=w-(x+26); i >= x; i-=DOTTEDLINE_WIDTH)
						 p->drawPoint(i, y+1);
				return;
			}

			if (active && !disabled)
			{
				p->setBrush(cg.highlight());
				p->fillRect(x, y, w, h, cg.highlight());
				p->setPen(NoPen);
				
				const QCOORD corners[] = { x+1, y2-1, x+1, y+1, x2-1, y+1, x2-1, y2-1 };
				p->drawConvexPolygon(QPointArray(4, corners));
				p->setPen(cg.highlightedText());
			}

			if (mi->iconSet())
			{
				p->save();

				QIconSet::Mode mode = disabled ? QIconSet::Disabled : QIconSet::Normal;
				QPixmap pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
				int pixw = pixmap.width();
				int pixh = pixmap.height();

				QRect cr(xpos, y, opt.maxIconWidth(), h);
				QRect pmr(0, 0, pixw, pixh);
				pmr.moveCenter(cr.center());

				if (reverseLayout)
					pmr = visualRect( pmr, r );

				p->setPen(cg.highlightedText());
				p->drawPixmap(pmr.topLeft(), pixmap);

				p->restore();
			}

			p->setPen(active ? cg.highlightedText() : cg.buttonText());

			xpos += xm;

			if (mi->custom())
			{
				int m = itemVMargin;
				p->setPen(cg.foreground());
				mi->custom()->paint(p, cg, active, !disabled, x+xm, y+m, w-xm-tab+1, h-2*m);
				return;
			}
			else
			{
				QString s = mi->text();
				if(!s.isNull())
				{
					int t = s.find('\t');
					int m = itemVMargin;

					int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;

					// set our text color
					if (active && !disabled)
						textcolor = cg.highlightedText();
					else if (disabled)
						textcolor = cg.mid().dark(110);
					else
						textcolor = cg.buttonText();

					// we have a shortcut on the right side of the menu
					if (t >= 0)
					{
						int xp;
						xp = x + w - tab - rightBorder - itemHMargin - itemFrame + 1;

						QRect rr = QRect(xp, y+m, tab, h-(2*m));
						if (reverseLayout)
							rr = visualRect(rr, r);

						//draw etched text if needed
						if (etchtext && disabled)
						{
							QRect etchedRect = QRect(rr);
							etchedRect.addCoords(1,1,1,1);
							p->setPen(cg.light());
							p->drawText(etchedRect, text_flags, s.mid(t+1));
						}

						p->setPen(textcolor);
						p->drawText(rr, text_flags, s.mid(t+1));
						s = s.left(t);
					}

					QRect rr = QRect(xpos, y+m, w-xm-tab+1, h-(2*m));
					if (reverseLayout) {
						rr = visualRect(rr, r);
						text_flags |= AlignRight;
					}

					//draw etched text if needed
					if (etchtext && disabled)
					{
						QRect etchedRect = QRect(rr);
						etchedRect.addCoords(1,1,1,1);
						p->setPen(cg.light());
						p->drawText(etchedRect, text_flags, s);
					}

					p->setPen(textcolor);
					p->drawText(rr, text_flags, s);
				}
				else if (mi->pixmap())
				{
					QPixmap *pixmap = mi->pixmap();
					if (pixmap->depth() == 1) 
						p->setBackgroundMode(OpaqueMode);
					p->drawPixmap(xpos, y, *pixmap);
					if (pixmap->depth() == 1)
						p->setBackgroundMode(TransparentMode);
				}
			}

			// a submenu
			if (mi->popup())
			{
				int dim = pixelMetric(PM_MenuButtonIndicator);

				xpos = x+w - dim - 2;

				if (active && !disabled)
					p->setPen(cg.highlightedText());
				else
					p->setPen(cg.text());


				QRect rr = QRect(xpos, y + h/2 - dim/2, dim, dim);
				if (reverseLayout)
					rr = visualRect( rr, r );
				drawPrimitive((reverseLayout ? PE_ArrowLeft : PE_ArrowRight), p, rr, cg, Style_Enabled);
			}

			if (checkable)
			{
				if (mi->isChecked())
				{
					int xp = x;

					SFlags cflags = Style_Default;

					cflags |= Style_On;
					if (!disabled)
						cflags |= Style_Enabled;

					QRect rr = QRect( xp + 3, y + 3, 16, h - 5 );
					if (reverseLayout)
						rr = visualRect( rr, r );

					drawPrimitive(PE_CheckMark, p, rr, cg, cflags);
				}
			}

			break;
		}

		// Menu and dockwindow empty space
		//
		case CE_DockWindowEmptyArea:
		case CE_MenuBarEmptyArea:
			p->fillRect(r, cg.button());
			break;
		default:
			winstyle->drawControl(element, p, widget, r, cg, flags, opt);
	}
}

void KonxStyle::drawComplexControl(ComplexControl control,
                                     QPainter *p,
                                     const QWidget *widget,
                                     const QRect &r,
                                     const QColorGroup &cg,
                                     SFlags flags,
                                     SCFlags controls,
                                     SCFlags active,
                                     const QStyleOption& opt) const
{
	bool enabled = flags & Style_Enabled;
	bool focused = flags & Style_HasFocus;

	switch(control)
	{
		//	LISTVIEW
		//	--------
		case CC_ListView:
		{
			if ( controls & SC_ListView )
			{
//                 renderPanel(p, r, cg, true, true, enabled, focused);
				QCommonStyle::drawComplexControl( control, p, widget, r, cg, flags, controls, active, opt );
//                 QRect rw(opt.rect());
//                 rw.addCoords(2,-2,1,1);
//                 p->setPen (Qt::blue);
//                 p->drawRect(rw);
			}

			if ( controls & ( SC_ListViewBranch | SC_ListViewExpand ) )
			{
				if (opt.isDefault())
				break;

				QListViewItem	*item = opt.listViewItem(),
								*child = item->firstChild();

				int y = r.y();
				int c;
				if ( active == SC_All && controls == SC_ListViewExpand ) 
					c = 2;
				else
				{
					int linebot = 0;

					// skip the stuff above the exposed rectangle
					while ( child && y + child->height() <= 0 )
					{
						y += child->totalHeight();
						child = child->nextSibling();
					}

					int bx = r.width() / 2;

					// paint stuff in the magical area
					QListView* v = item->listView();
					while ( child && y < r.height() )
					{
						int lh;
						if ( !item->multiLinesEnabled() )
							lh = child ? child->height() : 0;
						else
							lh = p->fontMetrics().height() + 2 * v->itemMargin();
						lh = QMAX( lh, QApplication::globalStrut().height() );
						if ( lh % 2 > 0 )
							lh++;
						linebot = y + lh/2;

						if ( (child->isExpandable() || child->childCount()) && (child->height() > 0) )
						{
							p->setPen(cg.dark());
							QPointArray a;
							if ( !child->isOpen() ) // closed
								a.setPoints(7, r_arrow);
							else //or open
								a.setPoints(7, d_arrow);

							a.translate(bx, linebot);
							p->drawLineSegments(a, 0, 3);
							p->drawPoint(a[6]);
						}
						y += child->totalHeight();
						child = child->nextSibling();
					}
				}
				p->setPen(cg.text());
			}
			break;
		}

	
		//	COMBOBOX
		//	--------
		case CC_ComboBox:
		{
			int x, y, w, h;
			const QComboBox *cb = (const QComboBox *)widget;
			r.rect(&x, &y, &w, &h);

			if (active & Style_Sunken)
				flags |= Style_Sunken;

			static const unsigned char downarrow_bits[] =
			{
				0x7f, 0xbe, 0x9c, 0x08, 0x00, 0x00, 0x00, 0x28,
				0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xb8
			};

			static const unsigned int handle_width = 16;
			static const unsigned int handle_offset = handle_width + 1;

			QBitmap downArrow = QBitmap(7, 4, downarrow_bits, true);
			downArrow.setMask(downArrow);

			int fw = pixelMetric(PM_DefaultFrameWidth, widget);
			if (flags & Style_Enabled)
				cb->editable() ? p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.base()) :
					p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.midlight());
			else
				p->fillRect(x + fw, y + fw, w - 2*fw, h - 2*fw, cg.button());

			renderPanel(p, r, cg, true, true, enabled, focused);

			p->save();
			
/*			Temporarily disabled - gray backgroud behind combo box arrows
 			p->setBrush(cg.button());
			p->setPen( NoPen);

			QRect hr = QRect(w - handle_offset, y + 1, handle_width, h - 2);
			if (reverseLayout) { hr = visualRect(hr, r); }

			p->drawRect(hr);
*/
			// draw the arrow depending whether the combo box is opened or not
			bool opened = cb->listBox() ? cb->listBox()->isVisible() : false;

			QRect cr(w - handle_offset + 1, (opened ? y + 1 : y), handle_width, h - 2);
			QRect pmr(0, 0, 7, 4);
			pmr.moveCenter(cr.center());
			if (reverseLayout) {
				pmr = visualRect( pmr, r );
			}

			//p->drawPixmap(pmr.topLeft(), downArrow);
			drawPrimitive((opened ? PE_ArrowDown : PE_ArrowLeft), p, pmr, cg, Style_Enabled);

			// draw focus rectangle
			if (focused)
			{
				QRect focusRect = visualRect( subRect( SR_ComboBoxFocusRect, widget ), widget );
				drawPrimitive( PE_FocusRect, p, focusRect, cg, flags);
			}

			p->restore();
			break;
		}
		
		//  SLIDER
		//  ------
		case CC_Slider:
		{
			QRect groove = visualRect (querySubControlMetrics(CC_Slider, widget, SC_SliderGroove, opt), widget);
			QRect handle = visualRect (querySubControlMetrics(CC_Slider, widget, SC_SliderHandle, opt), widget);

			// draw the 'base' (=groove) of the slider
			if ((controls & SC_SliderGroove) && groove.isValid())
			{
				drawKStylePrimitive ( KPE_SliderGroove, p, widget, groove, cg, flags );

				// draw focus rectangle
				if (flags & Style_HasFocus)
				{
					QRect focusRect = visualRect( subRect( SR_SliderFocusRect, widget ), widget );
					drawPrimitive( PE_FocusRect, p, focusRect, cg, flags);
				}
			}

			// and the handle
			if ((controls & SC_SliderHandle) && handle.isValid())
                 drawKStylePrimitive ( KPE_SliderHandle, p, widget, handle, cg, flags );

			// tickmarks
			if (controls & SC_SliderTickmarks)
				QCommonStyle::drawComplexControl(control, p, widget, r, cg, flags,
						SC_SliderTickmarks, active, opt );
			break;
		}

		//	TOOLBUTTON
		//	----------
		case CC_ToolButton:
		{
			const QToolButton *tb = (const QToolButton *) widget;

			QRect button, menuarea;
			button   = querySubControlMetrics(control, widget, SC_ToolButton, opt);
			menuarea = querySubControlMetrics(control, widget, SC_ToolButtonMenu, opt);

			SFlags bflags = flags,
			       mflags = flags;

			if (active & SC_ToolButton)
				bflags |= Style_Down;

			if (active & SC_ToolButtonMenu)
				mflags |= Style_Down;

			if (controls & SC_ToolButton)
			{
				//	If we're pressed, on, or raised...
				if (bflags & (Style_Down | Style_On | Style_Raised))
				{
					drawPrimitive(PE_ButtonTool, p, button, cg, bflags, opt);
				}
				else if (tb->parentWidget() && tb->parentWidget()->backgroundPixmap() &&
				           !tb->parentWidget()->backgroundPixmap()->isNull())
				{
					QPixmap pixmap = *(tb->parentWidget()->backgroundPixmap());
					p->drawTiledPixmap( r, pixmap, tb->pos() );
				}
			}

			// Draw a toolbutton menu indicator if required
			if (controls & SC_ToolButtonMenu)
			{
				if (mflags & (Style_Down | Style_On | Style_Raised)) 
					drawPrimitive(PE_ButtonDropDown, p, menuarea, cg, mflags, opt);

				drawPrimitive(PE_ArrowDown, p, menuarea, cg, mflags, opt);
			}

			if (tb->hasFocus() && !tb->focusProxy())
			{
				QRect fr = tb->rect();
				fr.addCoords(2, 2, -2, -2);
				drawPrimitive(PE_FocusRect, p, fr, cg);
			}

			break;
		}

		//	SPINWIDGETS
		//	-----------
		case CC_SpinWidget:
		{
			const QSpinWidget *sw = (const QSpinWidget *) widget;
			SFlags flags;
			PrimitiveElement pe;

			QRect swf = querySubControlMetrics(control, sw, SC_SpinWidgetFrame);
			QRect swbf = querySubControlMetrics(control, sw, SC_SpinWidgetButtonField);

			if (controls & SC_SpinWidgetFrame) // Don't draw the WindowsStyle thick frame.
				renderPanel(p, swf, cg, true, true, enabled, focused);

			if (controls & SC_SpinWidgetButtonField) 
			{
				p->save();

				p->setPen (NoPen);
				p->setBrush((sw->isEnabled()) ? cg.base() : cg.background());
				p->drawRect(swbf);

				p->restore();
			}

			if (controls & SC_SpinWidgetUp)
			{
				flags = Style_Default | Style_Enabled;
				if (active == SC_SpinWidgetUp)
				{
					flags |= Style_On;
					flags |= Style_Sunken;
				}
				else
					flags |= Style_Raised;

				if (sw->buttonSymbols() == QSpinWidget::PlusMinus)
					pe = PE_SpinWidgetPlus;
				else
					pe = PE_SpinWidgetUp;

				QRect re = sw->upRect();
				QColorGroup ucg = sw->isUpEnabled() ? cg : sw->palette().disabled();
				QColor col;
				if (!(sw->isEnabled()))
					col = ucg.background();
				else if (flags & Style_Raised)
					col = ucg.base();
				else
					col = ucg.highlight();
					
//                 p->fillRect(re.x() + 1, re.y() + 1, re.width() - 2, re.height() - 1, flags & Style_Raised ? ucg.base() : ucg.highlight());
				p->fillRect(re.x() + 1, re.y() + 1, re.width() - 2, re.height() - 1, col);
				re.setX(re.x() + 1);
				p->setPen(flags & Style_Raised ? ucg.foreground() : ucg.highlightedText());
				drawPrimitive(pe, p, re, ucg, flags);
			}

			if (controls & SC_SpinWidgetDown)
			{
				flags = Style_Default | Style_Enabled;
				if (active == SC_SpinWidgetDown)
				{
					flags |= Style_On;
					flags |= Style_Sunken;
				}
				else
					flags |= Style_Raised;

				if (sw->buttonSymbols() == QSpinWidget::PlusMinus)
					pe = PE_SpinWidgetMinus;
				else
					pe = PE_SpinWidgetDown;

				QRect re = sw->downRect();
				QColorGroup dcg = sw->isDownEnabled() ? cg : sw->palette().disabled();
				QColor col;
				if (!(sw->isEnabled()))
					col = dcg.background();
				else if (flags & Style_Raised)
					col = dcg.base();
				else
					col = dcg.highlight();
					
				p->fillRect(re.x() + 1, re.y(), re.width() - 2, re.height() - 1, col);
				p->setPen(flags & Style_Raised ? dcg.foreground() : dcg.highlightedText());
				re.setX(re.x() + 1);
				drawPrimitive(pe, p, re, dcg, flags);
			}

			break;
		}

		default:
			KStyle::drawComplexControl(control, p, widget, r, cg, flags, controls, active, opt);
			break;
	}
}


QRect KonxStyle::subRect(SubRect r, const QWidget *widget) const
{
	// Don't use KStyles progressbar subrect
	switch (r)
	{
		case SR_ComboBoxFocusRect:
		{
			QRect focusRect = querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxEditField );
			focusRect.addCoords(1,1,-1,-1);
			return focusRect;
		}
		case SR_SliderFocusRect:
		{
			QRect focusRect = widget->rect();
			focusRect.addCoords(2,2,-2,-2);
			return focusRect;
		}

		case SR_ProgressBarGroove:
		case SR_ProgressBarContents:
		case SR_ProgressBarLabel:
		{
			return winstyle->subRect(r, widget);
		}
		default: {
			return KStyle::subRect(r, widget);
		}
	}
}

QRect KonxStyle::querySubControlMetrics(ComplexControl control,
                                          const QWidget *widget,
                                          SubControl subcontrol,
                                          const QStyleOption &opt) const
{
	if (!widget)
		return QRect();

	QRect r(widget->rect());
	switch (control)
	{
		case CC_ComboBox:
		{
			switch (subcontrol)
			{
				case SC_ComboBoxEditField: 
					return QRect(r.x() + 2, r.y() + 2, r.width() - 19, r.height() - 4);

				case SC_ComboBoxListBoxPopup: 
				{
					QRect list = opt.rect();
					list.addCoords(1,-1,-1,-1);
					return list;
				}

				default: 
					return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
			}
			break;
		}

		case CC_SpinWidget:
		{
			int fw = pixelMetric(PM_SpinBoxFrameWidth, widget);
			QSize bs;
			bs.setHeight(QMAX(8, widget->height()/2));
			bs.setWidth(12);

			int y = fw;
			int x, lx, rx;
			x = widget->width() - y - bs.width();
			lx = fw;
			rx = x - fw;

			switch (subcontrol)
			{
				case SC_SpinWidgetUp: 
					return QRect(x, y+1, bs.width(), bs.height());

				case SC_SpinWidgetDown:
					return QRect(x, y + bs.height()-1, bs.width(), bs.height()-1);

				case SC_SpinWidgetFrame:
					return QRect(0, 0, widget->width(), widget->height());

				case SC_SpinWidgetEditField: 
					return QRect(lx, fw, widget->width() - (bs.width() + fw), widget->height() - 2 * fw);

				case SC_SpinWidgetButtonField:
					return QRect(x, y, bs.width(), widget->height() - 2 * fw);
			}
			break;
		}

		case CC_Slider:
		{
			switch (subcontrol)
			{
				case SC_SliderHandle:
					QRect handle = KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
					handle.addCoords(1,1,-1,-1);
					return handle;
			}
		}
		
		default:
		{
		}
	}
	return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
}

int KonxStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
	switch(m)
	{
		//	TABS
		//	----
		case PM_TabBarTabVSpace:
		{
			const QTabBar * tb = (const QTabBar *) widget;
			if (tb->shape() == QTabBar::RoundedAbove ||
			    tb->shape() == QTabBar::RoundedBelow)
				return 9;
			else
				return 0;
		}

		case PM_TabBarTabOverlap:
		{
			const QTabBar* tb = (const QTabBar*)widget;
			if (tb->shape() != QTabBar::RoundedAbove &&
			    tb->shape() != QTabBar::RoundedBelow)
				return 3;                   // Leave standard size alone
			else
				return 1;                   // Change size for our tabs only
		}

		//	SPLITTERS
		//	---------
		case PM_SplitterWidth:
		case PM_DockWindowHandleExtent:
			return 9;

		//	PROGRESSBARS
		//	------------
		case PM_ProgressBarChunkWidth:
			return 9;

		//	SLIDER
		//	------
		case PM_SliderLength:
			return 16;

		//	MENU INDICATOR
		//	--------------
		case PM_MenuButtonIndicator:
			return 8;

		//	HEADER INDICATOR
		//	----------------
		case PM_HeaderMarkSize:
		case PM_HeaderGripMargin:
			return 5;

		//	CHECKBOXES / RADIO BUTTONS
		//	--------------------------
		case PM_ExclusiveIndicatorWidth:    // Radiobutton size
		case PM_ExclusiveIndicatorHeight:   // 13x13
		case PM_IndicatorWidth:             // Checkbox size
		case PM_IndicatorHeight:            // 13x13
			return 13;

		//	FRAMES
		//	------
		case PM_SpinBoxFrameWidth:
			return 2;

		case PM_DockWindowFrameWidth:
		case PM_MenuBarFrameWidth:
			return 0;

		case PM_DefaultFrameWidth:
		{
//             if	(	widget && (	widget->inherits("QButton") ||
//                                  widget->inherits("QWidgetStack") || // for correct painting of QTabBars
//                                 widget->inherits("QLineEdit") ||
//                                 widget->inherits("QListView") ||
//                                 widget->inherits("QComboBox")
//                               ) 
//                 )
			if ( widget && ( widget->inherits ("QPopupMenu") ) )
				return 1;
			else
				return 2;
		}

		case PM_ButtonDefaultIndicator: 
		case PM_ButtonShiftVertical:
		case PM_ButtonShiftHorizontal:
			return 1;

		// after this amount, jump back to the original scroll bar position
		case PM_MaximumDragDistance:
			return 200;

		default:
			return KStyle::pixelMetric(m, widget);
	}
}


QSize KonxStyle::sizeFromContents(ContentsType t,
                                    const QWidget *widget,
                                    const QSize &s,
                                    const QStyleOption &opt) const
{
	switch (t)
	{
		case CT_PopupMenuItem:
		{
			if (!widget || opt.isDefault())
				break;

			const QPopupMenu *popup = (const QPopupMenu *)widget;
			QMenuItem *mi = opt.menuItem();
			int maxpmw = opt.maxIconWidth();
			int w = s.width(), h = s.height();
			bool checkable = popup->isCheckable();

			if (mi->custom())
			{
				w = mi->custom()->sizeHint().width();
				h = mi->custom()->sizeHint().height();
			}
			else if (mi->widget())
			{
				// don't change the size in this case.
			}
			else if (mi->isSeparator())
			{
				w = 10;
				h = 3;
			}
			else
			{
				if (mi->pixmap()) 
					h = QMAX(h, mi->pixmap()->height() + 2); //4

				h = QMAX(h, widget->fontMetrics().height() + 4); //10

				if (mi->iconSet()) 
					h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal).height() + 2); //8
			}

			if (!mi->text().isNull())
			{
				if ((mi->text().find('\t') >= 0) || mi->popup()) 
					w += 20;
			}

			if (maxpmw) 
				w += maxpmw;

			if (checkable && maxpmw < 20) 
				w += 20 - maxpmw;

			if (checkable || maxpmw > 0) 
				w += 2;

			w += 20;

			return QSize(w, h);
		}

		case CT_PushButton:
	    {
			const QPushButton* btn = static_cast<const QPushButton*>(widget);

			int w = s.width() + 2 * pixelMetric(PM_ButtonMargin, widget);
			int h = s.height() + 2 * pixelMetric(PM_ButtonMargin, widget);
			if ( btn->isDefault() || btn->autoDefault() )
			{
				w += 2* PM_ButtonDefaultIndicator;
				h += 2* PM_ButtonDefaultIndicator;
			}
			if ( btn->text().isEmpty() && s.width() < 32 )
				return QSize(w, h);

			return QSize( w+5, h );
		}

		case CT_ComboBox:
		{
			int arrow = 21;
			int fw = pixelMetric(PM_DefaultFrameWidth, widget);
			return QSize( s.width() + arrow, QMAX(s.height() + 2*fw, 16));
		}

		default:
			return KStyle::sizeFromContents (t, widget, s, opt);
	}

	return KStyle::sizeFromContents (t, widget, s, opt);
}

int KonxStyle::styleHint(StyleHint stylehint,
						   const QWidget *widget,
						   const QStyleOption &option,
						   QStyleHintReturn* returnData ) const
{
	int ret;

	switch (stylehint)
	{
		case SH_EtchDisabledText:
		case SH_Slider_SnapToValue:
		case SH_PrintDialog_RightAlignButtons:
		case SH_FontDialog_SelectAssociatedText:
		case SH_PopupMenu_AllowActiveAndDisabled:
		case SH_MenuBar_MouseTracking:
		case SH_PopupMenu_MouseTracking:
		case SH_ComboBox_ListMouseTracking:
		case SH_ScrollBar_MiddleClickAbsolutePosition:
		case SH_UnderlineAccelerator:
		case SH_MenuBar_AltKeyNavigation:
			ret = 1;
			break;

		case SH_MainWindow_SpaceBelowMenuBar:
		case SH_PopupMenu_SubMenuPopupDelay:
			ret = 0;
			break;

		case SH_ScrollBar_BackgroundMode:
			ret = NoBackground;
			break;

		default:
			ret = QCommonStyle::styleHint(stylehint, widget, option, returnData);
//             ret = winstyle->styleHint(stylehint, widget, option, returnData);
			break;
	}

	return ret;
}

// DISABLED FOR NOW //
/*
 
//
// void KonxStyle::paletteChanged()
// {
// 	QMenuBar *menuBar = 0L;
// 	QToolBar *toolBar = 0L;
// 	QWidget  *widget  = 0L;
// 
// 	QValueListIterator<QWidget*> it = m_widgets.begin();
// 	for ( ; it != m_widgets.end(); ++it ) {
// 		widget = *it;
// 		if ((toolBar = dynamic_cast<QToolBar*>( widget )))
// 			updatePalette( toolBar );
// 		else if ((menuBar = dynamic_cast<QMenuBar*>( widget )))
// 			updatePalette( menuBar );
// 	}
// }
// 
// void KonxStyle::updatePalette( QToolBar *bar )
// {
// 	QPalette pal = QApplication::palette();
// 	pal.setColor(QColorGroup::Button,
// 	             pal.active().color(QColorGroup::Background));
// 	bar->setPalette(pal);
// }
// 
// void KonxStyle::updatePalette( QMenuBar *bar )
// {
// 	QPalette pal = QApplication::palette();
// 	pal.setColor(QColorGroup::Button,
// 	             pal.active().color(QColorGroup::Background));
// 	bar->setPalette(pal);
// }
// 
// void KonxStyle::slotDestroyed()
// {
// 	m_widgets.remove( (QWidget*) sender() );
// }
// 
*/
bool KonxStyle::eventFilter(QObject *object, QEvent *event)
{
	if (KStyle::eventFilter( object, event))
		return true;

	if ( !object->isWidgetType() )
		return false;

	// focus highlight
    if ( ::qt_cast<QLineEdit*>(object) ||
		 ::qt_cast<QScrollView*>(object))
	{
        QWidget* widget = static_cast<QWidget*>(object);

#if 0
        if ( ::qt_cast<QSpinWidget*>(widget->parentWidget()) )
        {
            QWidget* spinbox = widget->parentWidget();
            if ((event->type() == QEvent::FocusIn) || (event->type() == QEvent::FocusOut))
            {
                spinbox->repaint(false);
            }
            return false;
        }
#endif

        if ((event->type() == QEvent::FocusIn) || (event->type() == QEvent::FocusOut))
        {
            widget->repaint(false);
        }
        return false;
    }

#if 0
	//Toolbar background gradient handling
	else if (event->type() == QEvent::Paint &&
			 object->parent() && !qstrcmp(object->name(), kdeToolbarWidget) )
	{
		// Draw a gradient background for custom widgets in the toolbar
		// that have specified a "kde toolbar widget" name.
		renderToolbarWidgetBackground(0, static_cast<QWidget*>(object));

		return false;	// Now draw the contents
	}
	else if (event->type() == QEvent::Paint  &&  object->parent() && ::qt_cast<QToolBar*>(object->parent()) 
			&& !::qt_cast<QPopupMenu*>(object) )
	{
		// We need to override the paint event to draw a
		// gradient on a QToolBarExtensionWidget.
		QToolBar* toolbar = static_cast<QToolBar*>(object->parent());
		QWidget* widget = static_cast<QWidget*>(object);
		QRect wr = widget->rect (), tr = toolbar->rect();
		QPainter p( widget );

		// FIXME: temp disabled;
		return true;
		renderStripedBackground(&p, wr, widget->colorGroup());
		
		//Draw terminator line, too
		p.setPen( toolbar->colorGroup().mid() );
		if ( toolbar->orientation() == Qt::Horizontal )
			p.drawLine( wr.width()-1, 0, wr.width()-1, wr.height()-1 );
		else
			p.drawLine( 0, wr.height()-1, wr.width()-1, wr.height()-1 );
		return true;

	}
#endif

	return false;
}

// vim: noet ts=4 sw=4
